@@ -14,7 +14,7 @@
 ;; GNU General Public License for more details.
 ;;
 ;; You should have received a copy of the GNU General Public License
-;; along with the GNU Shepherd.  If not, see <http://www.gnu.org/licenses/>.
+;; along with the GNU Shepherd.  If not, see <https://www.gnu.org/licenses/>.
 
 (define-module (shepherd logger)
   #:use-module ((fibers)
@@ -119,13 +119,22 @@ exceptions.  Return @var{now}."
 
 (define* (open-log-file file #:optional (mode #o640))
   "Open @var{file} as a log file, close-on-exec, in append mode, with UTF-8
-encoding, etc."
-  (let* ((fd     (open-fdes file (logior O_CREAT O_WRONLY O_APPEND O_CLOEXEC)
-                            mode))
-         (output (fdopen fd "al")))
-    (set-port-encoding! output "UTF-8")
-    (set-port-conversion-strategy! output 'substitute)
-    output))
+encoding, etc.  Create the parent directory of @var{file} if it does not
+already exist."
+  (catch 'system-error
+    (lambda ()
+      (let* ((fd     (open-fdes file (logior O_CREAT O_WRONLY O_APPEND O_CLOEXEC)
+                                mode))
+             (output (fdopen fd "al")))
+        (set-port-encoding! output "UTF-8")
+        (set-port-conversion-strategy! output 'substitute)
+        output))
+    (lambda args
+      (if (= ENOENT (system-error-errno args))
+          (begin
+            (mkdir-p (dirname file))
+            (open-log-file file mode))
+          (apply throw args)))))
 
 (define* (rotate-and-reopen-log-file output file rotated-file
                                      #:optional (mode #o640))
@@ -235,19 +244,9 @@ to @var{file}.  Assume it's logging for @var{service}."
   (let ((file (if (string-prefix? "/" file)
                   file
                   (in-vicinity (getcwd) file))))
-    (catch 'system-error
-      (lambda ()
-        (%service-file-logger channel file input
-                              #:service service
-                              #:history-size history-size))
-      (lambda args
-        (if (= ENOENT (system-error-errno args))
-            (begin
-              (mkdir-p (dirname file))
-              (%service-file-logger channel file input
-                                    #:service service
-                                    #:history-size history-size))
-            (apply throw args))))))
+    (%service-file-logger channel file input
+                          #:service service
+                          #:history-size history-size)))
 
 (define* (spawn-service-file-logger file input
                                     #:key
